import datetime

from django.urls import reverse

from functions import create_subject, create_visit, create_appointment, create_worker
from web.forms import AppointmentEditForm, SubjectEditForm
from web.models import Appointment, Subject
from web.views.notifications import get_today_midnight_date
from . import LoggedInTestCase


class AppointmentsViewTests(LoggedInTestCase):
    def test_appointments_list_request(self):
        response = self.client.get(reverse('web.views.appointments'))
        self.assertEqual(response.status_code, 200)

    def test_appointments_edit(self):
        subject = create_subject()
        visit = create_visit(subject)
        appointment = create_appointment(visit, when=datetime.datetime.now())
        create_worker(self.user, True)
        new_comment = 'new comment'
        new_status = appointment.APPOINTMENT_STATUS_NO_SHOW
        new_last_name = "new last name"
        form_appointment = AppointmentEditForm(user=self.user, instance=appointment, prefix="appointment")
        form_subject = SubjectEditForm(instance=subject, prefix="subject")
        form_data = {}
        for key, value in form_appointment.initial.items():
            if value is not None:
                form_data['appointment-{}'.format(key)] = value
        for key, value in form_subject.initial.items():
            if value is not None:
                form_data['subject-{}'.format(key)] = value
        form_data["appointment-comment"] = new_comment
        form_data["appointment-status"] = new_status
        form_data["subject-last_name"] = new_last_name
        response = self.client.post(
            reverse('web.views.appointment_edit', kwargs={'id': appointment.id}), data=form_data)
        self.assertEqual(response.status_code, 302)
        updated_appointment = Appointment.objects.filter(id=appointment.id)[0]
        updated_subject = Subject.objects.filter(id=subject.id)[0]
        self.assertEqual(new_comment, updated_appointment.comment)
        self.assertEqual(new_status, updated_appointment.status)
        self.assertEqual(new_last_name, updated_subject.last_name)

    def test_appointments_edit_without_visit(self):
        create_worker(self.user)

        appointment = create_appointment()
        appointment.visit = None
        appointment.save()

        response = self.client.get(
            reverse('web.views.appointment_edit', kwargs={'id': appointment.id}))
        self.assertEqual(response.status_code, 200)

    def test_save_appointments_edit_without_visit(self):
        create_worker(self.user)

        appointment = create_appointment()
        appointment.visit = None
        appointment.save()

        form_appointment = AppointmentEditForm(user=self.user, instance=appointment, prefix="appointment")
        form_data = {}
        for key, value in form_appointment.initial.items():
            if value is not None:
                form_data['appointment-{}'.format(key)] = value
        response = self.client.post(
            reverse('web.views.appointment_edit', kwargs={'id': appointment.id}), data=form_data)

        self.assertEqual(response.status_code, 200)

    def test_save_appointments_edit(self):
        subject = create_subject()
        create_worker(self.user, True)
        visit = create_visit(subject)
        appointment = create_appointment(visit, get_today_midnight_date())

        form_appointment = AppointmentEditForm(user=self.user, instance=appointment, prefix="appointment")
        form_subject = SubjectEditForm(instance=subject, prefix="subject")
        form_data = {}
        for key, value in form_appointment.initial.items():
            if value is not None:
                form_data['appointment-{}'.format(key)] = value
        for key, value in form_subject.initial.items():
            if value is not None:
                form_data['subject-{}'.format(key)] = value
        form_data["appointment-status"] = Appointment.APPOINTMENT_STATUS_FINISHED

        response = self.client.post(
            reverse('web.views.appointment_edit', kwargs={'id': appointment.id}), data=form_data)

        self.assertEqual(response.status_code, 302)

        updated_subject = Subject.objects.get(id=subject.id)
        self.assertTrue(updated_subject.information_sent)