# coding=utf-8
import datetime

from django.contrib.auth.models import User, AnonymousUser
from django.db.models import Count, Case, When

from ..models import Worker, Subject, Visit, Appointment, Location


class NotificationCount(object):
    title = ""
    count = 0
    style = ""
    type = ''

    def __init__(self, title="Unknown", count=0, style="fa fa-users text-aqua", type='web.views.appointments'):
        self.title = title
        self.count = count
        self.style = style
        self.type = type


def get_exceeded_visit_notifications_count(user):
    notification = NotificationCount(
        title="exceeded visit time",
        count=get_exceeded_visits(user).count(),
        style="fa fa-thermometer-4 text-red",
        type='web.views.exceeded_visits')
    return notification


def get_subjects_with_reminder_count(user):
    notification = NotificationCount(
        title="subject required contact",
        count=get_subjects_with_reminder(user).count(),
        style="fa fa-users text-aqua",
        type='web.views.subject_require_contact')
    return notification


def get_subject_with_no_visit_notifications_count(user):
    notification = NotificationCount(
        title="subject without visit",
        count=get_subjects_with_no_visit(user).count(),
        style="fa fa-users text-aqua",
        type='web.views.subject_no_visits')
    return notification


def get_visits_without_appointments_count(user):
    notification = NotificationCount(
        title="unfinished visits ",
        count=len(get_unfinished_visits(user)),
        style="fa fa-user-times text-yellow",
        type='web.views.unfinished_visits')
    return notification


def get_visits_with_missing_appointments_count(user):
    notification = NotificationCount(
        title="visits with missing appointments",
        count=len(get_active_visits_with_missing_appointments(user)),
        style="fa fa-user-times text-yellow",
        type='web.views.visits_with_missing_appointments')
    return notification


def get_approaching_visits_without_appointments_count(user):
    notification = NotificationCount(
        title="approaching visits ",
        count=get_approaching_visits_without_appointments(user).count(),
        style="fa fa-users text-aqua",
        type='web.views.approaching_visits_without_appointments')
    return notification


def get_approaching_visits_for_mail_contact_count(user):
    notification = NotificationCount(
        title="post mail for approaching visits",
        count=get_approaching_visits_for_mail_contact(user).count(),
        style="fa fa-users text-aqua",
        type='web.views.approaching_visits_for_mail_contact')
    return notification


def get_unfinished_appointments_count(user):
    return NotificationCount(
        title="unfinished appointments ",
        count=get_unfinished_appointments(user).count(),
        style="fa fa-history text-yellow",
        type='web.views.unfinished_appointments')


def get_notifications(the_user):
    worker = Worker.get_by_user(the_user)
    notifications = []
    count = 0
    if worker is not None:
        notifications.append(get_exceeded_visit_notifications_count(worker))
        notifications.append(get_visits_without_appointments_count(worker))
        notifications.append(get_approaching_visits_without_appointments_count(worker))
        notifications.append(get_unfinished_appointments_count(worker))
        notifications.append(get_visits_with_missing_appointments_count(worker))
        notifications.append(get_subject_with_no_visit_notifications_count(worker))
        notifications.append(get_approaching_visits_for_mail_contact_count(worker))
        notifications.append(get_subjects_with_reminder_count(worker))

        for notification in notifications:
            count += notification.count
    return count, notifications


def get_subjects_with_no_visit(user):
    result = Subject.objects.annotate(my_count=Count(Case(When(visit__is_finished=False, then=1)))).filter(
        dead=False,
        resigned=False,
        my_count=0,
        default_location__in=get_filter_locations(user),
        postponed=False,
        datetime_contact_reminder__isnull=True,
    )
    return result


def get_subjects_with_reminder(user):
    tomorrow = datetime.datetime.now() + datetime.timedelta(hours=1)

    result = Subject.objects.filter(
        dead=False,
        resigned=False,
        default_location__in=get_filter_locations(user),
        datetime_contact_reminder__lt=tomorrow,
    )
    return result


def get_active_visits_with_missing_appointments(user):
    result = []
    for visit in get_active_visits_without_appointments(user):
        if waiting_for_appointment(visit):
            result.append(visit)
    return result


def get_unfinished_visits(user):
    result = []
    for visit in get_active_visits_without_appointments(user):
        if not waiting_for_appointment(visit):
            result.append(visit)
    return result


def get_approaching_visits_without_appointments(user):
    today = get_today_midnight_date()
    today_plus_two_months = today + datetime.timedelta(days=91)
    return Visit.objects.annotate(
        my_count=Count(Case(When(appointment__status=Appointment.APPOINTMENT_STATUS_SCHEDULED, then=1)))).filter(
        datetime_begin__gt=today,
        datetime_begin__lt=today_plus_two_months,
        is_finished=False,
        subject__default_location__in=get_filter_locations(user),
        my_count=0).order_by('datetime_begin')


def get_approaching_visits_for_mail_contact(user):
    today = get_today_midnight_date()
    today_plus_three_months = today + datetime.timedelta(days=91)
    today_plus_six_months = today + datetime.timedelta(days=183)
    return Visit.objects.annotate(
        my_count=Count(Case(When(appointment__status=Appointment.APPOINTMENT_STATUS_SCHEDULED, then=1)))).filter(
        datetime_begin__gt=today_plus_three_months,
        datetime_begin__lt=today_plus_six_months,
        is_finished=False,
        post_mail_sent=False,
        subject__default_location__in=get_filter_locations(user),
        my_count=0).order_by('datetime_begin')


def get_exceeded_visits(user):
    return Visit.objects.filter(datetime_end__lt=get_today_midnight_date(),
                                is_finished=False,
                                subject__default_location__in=get_filter_locations(user)
                                ).order_by('datetime_begin')


def get_unfinished_appointments(user):
    return Appointment.objects.filter(
        datetime_when__lt=get_today_midnight_date(),
        status=Appointment.APPOINTMENT_STATUS_SCHEDULED,
        location__in=get_filter_locations(user),
    ).order_by('datetime_when')


def waiting_for_appointment(visit):
    required_types = visit.appointment_types.all()
    appointment_types = []
    for appointment in visit.appointment_set.all():
        for type in appointment.appointment_types.all():
            if (appointment.status in [Appointment.APPOINTMENT_STATUS_FINISHED,
                                       Appointment.APPOINTMENT_STATUS_SCHEDULED]) and (not (type in appointment_types)):
                appointment_types.append(type)
    result = False
    for type in required_types:
        if not (type in appointment_types):
            result = True
    return result


def get_active_visits_without_appointments(user):
    today = get_today_midnight_date()
    return Visit.objects.annotate(
        my_count=Count(Case(When(appointment__status=Appointment.APPOINTMENT_STATUS_SCHEDULED, then=1)))).filter(
        datetime_begin__lt=today,
        datetime_end__gt=today,
        is_finished=False,
        subject__default_location__in=get_filter_locations(user),
        my_count=0).order_by('datetime_begin')


def get_filter_locations(user):
    worker = None

    if isinstance(user, User):
        workers = Worker.objects.filter(user=user)
        if len(workers) > 0:
            worker = workers[0]
    elif isinstance(user, Worker):
        worker = user
    elif isinstance(user, AnonymousUser):
        # anonymous user shouldn't see anything
        return Location.objects.filter(id=-1)
    elif user is not None:
        raise TypeError("Unknown class type: " + user.__class__.__name__)

    if worker is None or worker.locations.count() == 0:
        return Location.objects.all()
    else:
        return worker.locations.all()


def get_today_midnight_date():
    today = datetime.datetime.now()
    today_midnight = datetime.datetime(today.year, today.month, today.day)
    return today_midnight